from rest_framework.routers import DefaultRouter


class ExtendedRouter(DefaultRouter):
    def get_method_map(self, viewset, method_map):
        _method_map = super().get_method_map(viewset, method_map)

        if getattr(viewset, "is_singleton", False):
            _method_map["patch"] = "update"

        return _method_map


content_studio_router = ExtendedRouter(trailing_slash=False)



import uuid


class BaseExtension:
    extension_type: str
    extension_id: uuid.UUID

    def __init__(self):
        self.extension_id = uuid.uuid4()

    def serialize(self):
        return {
            "extension_type": self.extension_type,
            "extension_id": str(self.extension_id),
        }


class MainMenuLink(BaseExtension):
    extension_type = "MainMenuLink"
    url: str
    label: str
    icon: str = None
    color: str = None
    weight: int = 0

    def __init__(
        self, url: str, label: str, icon: str = None, color: str = None, weight: int = 0
    ):
        self.url = url
        self.label = label
        self.icon = icon
        self.color = color
        self.weight = weight
        super().__init__()

    def serialize(self):
        return {
            **super().serialize(),
            "config": {
                "url": self.url,
                "icon": self.icon,
                "color": self.color,
                "label": self.label,
            },
        }


class IFramePage(BaseExtension):
    extension_type = "IFramePage"
    path: str
    iframe_url: str

    def __init__(
        self,
        path: str,
        iframe_url: str,
    ):
        self.path = path
        self.iframe_url = iframe_url
        super().__init__()

    def serialize(self):
        return {
            **super().serialize(),
            "config": {
                "path": self.path,
                "iframe_url": self.iframe_url,
            },
        }



from rest_framework import serializers

from content_studio.settings import cs_settings


class MediaItemSerializer(serializers.ModelSerializer):
    thumbnail = serializers.SerializerMethodField()

    class Meta:
        model = cs_settings.MEDIA_LIBRARY_MODEL
        fields = "__all__"

    def get_thumbnail(self, obj):
        admin_site = cs_settings.ADMIN_SITE

        if admin_site and obj.type == "image":
            return admin_site.get_thumbnail(obj)

        return None


class MediaFolderSerializer(serializers.ModelSerializer):
    has_children = serializers.SerializerMethodField()

    class Meta:
        model = cs_settings.MEDIA_LIBRARY_FOLDER_MODEL
        fields = ["id", "name", "parent", "has_children"]

    def get_has_children(self, obj):
        return obj.children.exists()



###
# Formats determine how fields are displayed in Django Content Studio.
# Each Django model field has a default format, but this can be
# overridden in the admin model.
###


class BaseFormat:
    @classmethod
    def serialize(cls):
        return {"name": cls.__name__}


class ForeignKeyFormat(BaseFormat):
    pass


class MediaFormat(BaseFormat):
    pass


class ManyMediaFormat(BaseFormat):
    pass


class TextFormat(BaseFormat):
    pass


class HtmlFormat(BaseFormat):
    pass


class BooleanFormat(BaseFormat):
    pass


class DateFormat(BaseFormat):
    pass


class DateTimeFormat(BaseFormat):
    pass


class TimeFormat(BaseFormat):
    pass


class NumberFormat(BaseFormat):
    pass


class FileSizeFormat(BaseFormat):
    pass


class TagFormat(BaseFormat):
    pass


class JSONFormat(BaseFormat):
    pass



from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from ..dashboard import BaseWidget


class ScheduledTaskSerializer(serializers.Serializer):
    title = serializers.CharField()
    description = serializers.CharField(allow_blank=True, required=False)
    last_run_at = serializers.DateTimeField(allow_null=True)
    next_run_at = serializers.DateTimeField(allow_null=True)
    duration = serializers.IntegerField(allow_null=True)
    status = serializers.ChoiceField(
        choices=["RUNNING", "SCHEDULED", "SUCCESS", "FAILURE"]
    )
    error_message = serializers.CharField(
        allow_blank=True, allow_null=True, required=False
    )


class ScheduledTasksWidgetSerializer(serializers.Serializer):
    title = serializers.CharField(
        default=_("Scheduled tasks"), allow_blank=True, required=False
    )
    description = serializers.CharField(
        default=_("Monitor and manage automated tasks"),
        allow_blank=True,
        required=False,
    )
    tasks = serializers.ListField(child=ScheduledTaskSerializer())


class ScheduledTasksWidget(BaseWidget):
    """
    Widget for showing a list of scheduled tasks.
    """

    name = "ScheduledTasksWidget"

    col_span = 2

    def get_data(self, request):
        raise NotImplementedError("You need to implement get_data for your widget.")



import json
import sys

from rich.console import Console

from content_studio.settings import cs_settings

console = Console()


def log(*args, **kwargs):
    console.print(*args, **kwargs)


def flatten(xss):
    return [x for xs in xss for x in xs]


def is_runserver():
    """
    Checks if the Django application is started as a server.
    We'll also assume it started if manage.py is not used (e.g. when Django is started using wsgi/asgi).
    The main purpose of this check is to not run certain code on other management commands such
    as `migrate`.
    """
    is_manage_cmd = sys.argv[0].endswith("/manage.py")

    return not is_manage_cmd or sys.argv[1] == "runserver"


def is_jsonable(x):
    try:
        json.dumps(x)
        return True
    except (TypeError, OverflowError):
        return False


def get_related_field_name(inline, parent_model):
    """
    Get the name of the foreign key field in the inline model.
    """
    if inline.fk_name:
        return inline.fk_name

    # Let Django figure it out

    opts = inline.model._meta

    # Find all foreign keys pointing to parent model
    fks = [
        f
        for f in opts.get_fields()
        if f.many_to_one and f.remote_field.model == parent_model
    ]

    if len(fks) == 1:
        return fks[0].name
    elif len(fks) == 0:
        raise ValueError(
            f"No foreign key found in {inline.model} pointing to {parent_model}"
        )
    else:
        raise ValueError(f"Multiple foreign keys found. Specify fk_name on the inline.")


def get_tenant_field_name(model):
    tenant_model = cs_settings.TENANT_MODEL

    if not tenant_model:
        return None

    opts = model._meta

    # Find all foreign keys pointing to the tenant model
    fks = [
        f
        for f in opts.get_fields()
        if f.many_to_one and f.remote_field.model == tenant_model
    ]

    if len(fks) == 1:
        return fks[0].name
    elif len(fks) == 0:
        return None
    else:
        raise ValueError(
            f"Multiple fields found pointing to {tenant_model}. Only one field can point to a tenant model."
        )



from django.core.exceptions import ImproperlyConfigured

from .jwt import SimpleJwtBackend
from ..router import content_studio_router


class TokenBackendManager:
    """
    Manages different token authentication backends for use by
    Content Studio.

    While login backends are used to identify a user,
    token backends determine are used to authenticate
    communication between Content Studio and the admin API.
    """

    available_backends = [SimpleJwtBackend]
    _active_backend = None

    @property
    def active_backend(self):
        if self._active_backend:
            return self._active_backend

        for backend in self.available_backends:
            if backend.is_available:
                self._active_backend = backend
                return backend

        raise ImproperlyConfigured(
            "You need to install at least one of the support authentication backends."
        )

    def set_up_router(self):
        content_studio_router.register(
            f"api/tokens/{self.active_backend.__name__.lower().replace('backend', '')}",
            self.active_backend.view_set,
            basename="content_studio_token_backend",
        )



import datetime
import random

from django.db import models
from django.utils import timezone

from content_studio.settings import cs_settings


def generate_code():
    return "".join(str(random.randint(0, 9)) for _ in range(6))


class PasswordResetCode(models.Model):
    class Meta:
        db_table = "dcs_password_reset_code"
        verbose_name = "Password reset code"
        verbose_name_plural = "Password reset codes"

    email = models.EmailField(
        max_length=255, verbose_name="Email address", editable=False
    )

    code = models.CharField(
        max_length=6, verbose_name="Code", default=generate_code, editable=False
    )

    created_at = models.DateTimeField(auto_now_add=True, verbose_name="Created at")

    @property
    def expired(self):
        return (
            self.created_at
            + datetime.timedelta(minutes=cs_settings.PASSWORD_RESET_EXPIRATION_TIME)
            < timezone.now()
        )

    def __str__(self):
        return self.email



from django.contrib.admin.models import LogEntry
from rest_framework import serializers

from content_studio.serializers import ContentSerializer
from ..dashboard import BaseWidget


class LogEntrySerializer(ContentSerializer):
    object_model = serializers.SerializerMethodField()

    class Meta:
        model = LogEntry
        fields = [
            "id",
            "action_flag",
            "action_time",
            "user",
            "object_id",
            "object_repr",
            "object_model",
        ]

    def get_object_model(self, obj):
        return f"{obj.content_type.app_label}.{obj.content_type.model}"


class ActivityLogWidget(BaseWidget):
    """
    Widget for showing activity logs.
    """

    name = "ActivityLogWidget"

    col_span = 2

    def get_data(self, request):
        return LogEntrySerializer(
            LogEntry.objects.all().order_by("-action_time")[0:5], many=True
        ).data



###
# Form field classes are used for grouping, ordering and laying out fields.
###
import uuid
from typing import Type

from django.db.models import Model
from rest_framework.response import Response


class Field:
    """
    Field class for configuring the fields in content edit views in Django Content Studio.
    """

    def __init__(
        self, name: str, col_span: int = 1, label: str = None, readonly: bool = False
    ):
        self.name = name
        self.col_span = col_span
        self.label = label
        self.readonly = readonly

    def serialize(self):
        return {
            "type": "field",
            "name": self.name,
            "col_span": self.col_span,
            "label": self.label,
            "readonly": self.readonly,
        }


class FieldLayout:
    """
    Field layout class for configuring the layout of fields in content edit views in Django Content Studio.
    """

    def __init__(self, fields: list[str | Field] = None, columns: int = 1):
        self.fields = [self._normalize_field(f) for f in fields] if fields else []
        self.columns = columns

    def serialize(self):
        return {
            "type": "field-layout",
            "fields": [field.serialize() for field in self.fields],
            "columns": self.columns,
        }

    def get_fields(self) -> list[Field]:
        return self.fields

    def _normalize_field(self, field):
        """
        Checks if a field is of an allowed type and wraps string fields in a Field object.
        """
        if isinstance(field, str):
            return Field(field)
        elif isinstance(field, Field):
            return field
        else:
            raise ValueError(f"Invalid field: {field}. Must be a string or Field.")


class FormSet:
    """
    Formset class for configuring the blocks of fields in content edit views
    in Django Content Studio.
    """

    def __init__(
        self,
        title: str = "",
        description: str = "",
        fields: list[str | Field | FieldLayout] = None,
    ):
        self.title = title
        self.description = description
        self.fields = [self._normalize_field(f) for f in fields] if fields else []

    def serialize(self):
        return {
            "type": "form-set",
            "title": self.title,
            "description": self.description,
            "fields": [field.serialize() for field in self.fields],
        }

    def get_fields(self) -> list[Field]:
        """
        Returns a list of all Field and Component objects.
        """
        fields = []

        for field in self.fields:
            if isinstance(field, FieldLayout):
                fields = fields + field.get_fields()
            else:
                fields.append(field)

        return fields

    def _normalize_field(self, field):
        """
        Checks if a field is of an allowed type and wraps string fields in a Field object.
        """
        if isinstance(field, str):
            return Field(field)
        elif isinstance(field, Field):
            return field
        elif isinstance(field, FieldLayout):
            return field
        elif issubclass(field.__class__, Component):
            return field
        else:
            raise ValueError(
                f"Invalid field: {field}. Must be a string, Field, FieldLayout or a Component (subclass)."
            )


class FormSetGroup:
    """
    Formset group class for configuring the groups of form sets in content edit views.
    """

    def __init__(self, label: str = "", formsets: list[FormSet] = None):
        self.label = label
        self.formsets = formsets or []

    def serialize(self):
        return {
            "type": "form-set-group",
            "label": self.label,
            "formsets": [formset.serialize() for formset in self.formsets],
        }

    def get_fields(self) -> list[Field]:
        """
        Returns a list of all Field and Component objects.
        """
        fields = []

        for form_set in self.formsets:
            fields = fields + form_set.get_fields()

        return fields


class Component:
    component_id: uuid.UUID
    component_type: str

    def __init__(self):
        self.component_id = uuid.uuid4()

    def serialize(self):
        return {
            "type": "component",
            "component_type": str(self.component_type),
            "component_id": str(self.component_id),
        }


class Link(Component):
    component_type = "Link"
    label: str

    def get_url(self, obj: Type[Model], request):
        raise NotImplementedError

    def serialize(self):
        return {
            **super().serialize(),
            "label": self.label,
        }

    def handle_request(self, obj: Type[Model], request):
        return Response(data={"url": self.get_url(obj, request)})


class ButtonLink(Link):
    component_type = "LinkButton"

    # Add icon before label
    icon = None

    # Add copy to clipboard button
    copy = False

    def serialize(self):
        return {
            **super().serialize(),
            "icon": self.icon,
            "copy": self.copy,
        }



from django.conf import settings
from django.contrib import admin
from django.urls import reverse, NoReverseMatch
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.generic import TemplateView
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.exceptions import MethodNotAllowed
from rest_framework.permissions import IsAdminUser, AllowAny
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet

from . import __version__
from .admin import AdminSerializer, ModelGroup
from .models import ModelSerializer
from .serializers import SessionUserSerializer
from .settings import cs_settings


@method_decorator(never_cache, name="dispatch")
class ContentStudioWebAppView(TemplateView):
    """
    View for rendering the content studio web app.
    """

    template_name = "content_studio/index.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["version"] = __version__
        return context


class AdminApiViewSet(ViewSet):
    """
    View set for content studio admin endpoints.
    """

    permission_classes = [IsAdminUser]
    renderer_classes = [JSONRenderer]
    admin_site = cs_settings.ADMIN_SITE

    @action(
        methods=["get"],
        detail=False,
        url_path="info",
        permission_classes=[AllowAny],
    )
    def info(self, request):
        """
        Returns public information about the Content Studio admin.
        """

        data = {
            "version": __version__,
            "site_header": self.admin_site.site_header,
            "site_title": self.admin_site.site_title,
            "index_title": self.admin_site.index_title,
            "site_url": self.admin_site.site_url,
            "health_check": get_health_check_path(),
            "login_backends": [
                backend.get_info()
                for backend in self.admin_site.login_backend.active_backends
            ],
            "token_backend": self.admin_site.token_backend.active_backend.get_info(),
            "formats": {
                model_class.__name__: frmt.serialize()
                for model_class, frmt in self.admin_site.default_format_mapping.items()
            },
            "widgets": get_widgets(),
            "settings": {
                "created_by_attr": cs_settings.CREATED_BY_ATTR,
                "created_at_attr": cs_settings.CREATED_AT_ATTR,
                "edited_by_attr": cs_settings.EDITED_BY_ATTR,
                "edited_at_attr": cs_settings.EDITED_AT_ATTR,
            },
        }

        return Response(data=data)

    @action(
        methods=["get"],
        detail=False,
        url_path="discover",
    )
    def discover(
        self,
        request,
    ):
        """
        Returns information about the Django app (models, admin models, admin site, settings, etc.).
        """
        data = {
            "models": get_models(request),
            "model_groups": get_model_groups(),
            "user_model": settings.AUTH_USER_MODEL,
            "extensions": get_extensions(),
        }

        media_model = cs_settings.MEDIA_LIBRARY_MODEL
        folder_model = cs_settings.MEDIA_LIBRARY_FOLDER_MODEL

        data["media_library"] = {
            "enabled": media_model is not None,
            "folders": folder_model is not None,
            "models": {
                "media_model": media_model._meta.label_lower,
                "folder_model": folder_model._meta.label_lower,
            },
        }

        if self.admin_site.dashboard:
            data["dashboard"] = self.admin_site.dashboard.serialize()
        else:
            data["dashboard"] = {"widgets": []}

        multitenancy = cs_settings.TENANT_MODEL is not None

        data["multitenancy"] = {
            "enabled": multitenancy,
            "tenant_model": (
                cs_settings.TENANT_MODEL._meta.label_lower if multitenancy else None
            ),
        }

        return Response(data=data)

    @action(methods=["get"], detail=False, url_path="me")
    def me(self, request):
        """
        Returns information about the current user.
        """
        return Response(SessionUserSerializer(request.user).data)

    @action(
        methods=["get"],
        detail=False,
        url_path="tenants",
    )
    def list_tenants(self, request):
        tenant_model = cs_settings.TENANT_MODEL

        if not tenant_model:
            raise MethodNotAllowed("GET", "Tenant model not defined.")

        class TenantSerializer(serializers.ModelSerializer):
            class Meta:
                model = tenant_model
                fields = ["id", "__str__"]

        tenants = self.admin_site.get_tenants(
            tenant_model=tenant_model, request=request
        )

        return Response(TenantSerializer(tenants, many=True).data)


def get_models(request):
    models = []
    registered_models = admin.site._registry

    for model, admin_class in registered_models.items():
        models.append(
            {
                **ModelSerializer(model).serialize(),
                "admin": AdminSerializer(admin_class).serialize(request),
            }
        )
        for inline in admin_class.inlines:
            if inline.model not in registered_models:
                models.append(
                    {
                        **ModelSerializer(inline.model).serialize(),
                    }
                )

    return models


def get_model_groups():
    admin_site = cs_settings.ADMIN_SITE

    default_group = [
        ModelGroup(
            label=_("Content"),
            name="default",
            icon=None,
            models=[model for model, admin_model in admin.site._registry.items()],
        )
    ]
    # Get custom model groups or use the default one
    model_groups = getattr(admin_site, "model_groups", None) or default_group

    return [
        {
            "name": group.name,
            "icon": group.icon,
            "color": group.color,
            "label": group.label,
            "models": [m._meta.label_lower for m in group.models],
        }
        for group in model_groups
    ]


def get_widgets():
    admin_site = cs_settings.ADMIN_SITE

    return {
        (m if isinstance(m, str) else m.__name__): widget.serialize()
        for m, widget in admin_site.default_widget_mapping.items()
    }


def get_health_check_path():
    try:
        return reverse("healthcheck")
    except NoReverseMatch:
        return None


def get_extensions():
    admin_site = cs_settings.ADMIN_SITE

    return [i.serialize() for i in getattr(admin_site, "extensions", [])]



from rest_framework import serializers

from ..dashboard import BaseWidget


class StatisticWidgetSerializer(serializers.Serializer):
    value = serializers.IntegerField()
    prefix = serializers.CharField(default="", allow_blank=True, required=False)
    suffix = serializers.CharField(default="", allow_blank=True, required=False)
    title = serializers.CharField(default="", allow_blank=True, required=False)
    trend = serializers.DecimalField(
        max_digits=5, decimal_places=1, allow_null=True, required=False
    )
    trend_sentiment = serializers.ChoiceField(
        default="default", required=False, choices=["default", "positive", "negative"]
    )


class StatisticWidget(BaseWidget):
    """
    Widget for showing some statistic.
    """

    name = "StatisticWidget"

    def get_data(self, request):
        raise NotImplementedError("You need to implement get_data for your widget.")



from django.contrib.auth import get_user_model
from django.core.mail import send_mail
from django.utils.translation import gettext_lazy as _
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import PasswordResetCode
from .serializers import (
    PasswordResetRequestSerializer,
    CodeValidationSerializer,
    PasswordResetSubmissionSerializer,
)


class PasswordResetRequestView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        serializer = PasswordResetRequestSerializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        email = serializer.validated_data["email"]

        user_model = get_user_model()

        try:
            user_model.objects.get(email=email)
        except user_model.DoesNotExist:
            return Response(status=status.HTTP_202_ACCEPTED)

        # Delete any existing codes for this email.
        PasswordResetCode.objects.filter(email=email).delete()

        reset = PasswordResetCode.objects.create(email=email)

        try:
            self.send_email(reset)
        except Exception as e:
            reset.delete()
            print(e)

        return Response(status=status.HTTP_202_ACCEPTED)

    def send_email(self, reset: PasswordResetCode):
        send_mail(
            from_email=None,
            subject=_("Your password reset code"),
            message=_("Your password reset code is: ") + reset.code,
            recipient_list=[reset.email],
        )


class CodeValidationView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        serializer = CodeValidationSerializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        code = serializer.validated_data["code"]

        existing = PasswordResetCode.objects.filter(code=code).first()

        if not existing:
            raise ValidationError("Invalid code.")

        if existing.expired:
            existing.delete()
            raise ValidationError("Expired code.")

        return Response(status=status.HTTP_204_NO_CONTENT)


class PasswordResetSubmissionView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        serializer = PasswordResetSubmissionSerializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        code = serializer.validated_data["code"]
        email = serializer.validated_data["email"]
        password = serializer.validated_data["password"]

        existing = PasswordResetCode.objects.filter(code=code, email=email).first()

        if not existing:
            raise ValidationError("Invalid code.")

        if existing.expired:
            existing.delete()
            raise ValidationError("Expired code.")

        user_model = get_user_model()
        user = user_model.objects.filter(email=email).first()

        if not user:
            raise ValidationError("Invalid email.")

        user.set_password(password)

        user.save()

        existing.delete()

        try:
            self.send_email(email)
        except Exception as e:
            print(e)

        return Response(status=status.HTTP_204_NO_CONTENT)

    def send_email(self, email):
        send_mail(
            from_email=None,
            subject=_("Your password has been reset"),
            message=_(
                "Your password has been reset. If you did not do this, please contact your administrator."
            ),
            recipient_list=[email],
        )



from django.contrib.auth import get_user_model, authenticate
from django.utils import timezone
from rest_framework import serializers
from rest_framework.exceptions import PermissionDenied
from rest_framework.permissions import AllowAny
from rest_framework.renderers import JSONRenderer
from rest_framework.viewsets import ViewSet


class UsernamePasswordSerializer(serializers.Serializer):
    username = serializers.CharField(required=True)
    password = serializers.CharField(required=True)


class UsernamePasswordViewSet(ViewSet):
    """
    View set for username and password endpoints.
    """

    permission_classes = [AllowAny]
    renderer_classes = [JSONRenderer]

    def create(self, request):
        serializer = UsernamePasswordSerializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        user = UsernamePasswordBackend.login(
            username=serializer.validated_data["username"],
            password=serializer.validated_data["password"],
        )

        if user is None:
            raise PermissionDenied(detail="Invalid username or password.")

        if not user.is_active:
            raise PermissionDenied(detail="User account is disabled.")

        from ..admin import admin_site

        user.last_login = timezone.now()

        user.save()

        return admin_site.token_backend.active_backend.get_response_for_user(user)


class UsernamePasswordBackend:
    name = "Username password"
    view_set = UsernamePasswordViewSet

    @classmethod
    def get_info(cls):
        """
        Returns information about the backend.
        """
        user_model = get_user_model()
        username_field = getattr(user_model, user_model.USERNAME_FIELD, None)

        return {
            "type": cls.__name__,
            "config": {"username_field_type": username_field.field.__class__.__name__},
        }

    @classmethod
    def login(cls, username, password):
        """
        Authenticates user using username and password.
        Returns the user if successful, None otherwise.
        """
        return authenticate(username=username, password=password)



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class ContentPagination(PageNumberPagination):
    page_size = 20
    page_size_query_param = "limit"
    max_page_size = 100

    def get_paginated_response(self, data):
        return Response(
            {
                "pagination": {
                    "count": self.page.paginator.count,
                    "current": self.page.number,
                    "pages": self.page.paginator.num_pages,
                },
                "results": data,
            }
        )



# Generated by Django 6.0.1 on 2026-02-04 09:42

import content_studio.contrib.password_reset.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="PasswordResetCode",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        editable=False, max_length=255, verbose_name="Email address"
                    ),
                ),
                (
                    "code",
                    models.CharField(
                        default=content_studio.contrib.password_reset.models.generate_code,
                        editable=False,
                        max_length=6,
                        verbose_name="Code",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created at"),
                ),
            ],
            options={
                "verbose_name": "Password reset code",
                "verbose_name_plural": "Password reset codes",
                "db_table": "dcs_password_reset_code",
            },
        ),
    ]



import inspect

from django.contrib.auth import get_user_model
from rest_framework import serializers
from rest_framework.relations import RelatedField

user_model = get_user_model()


class ContentRelatedField(RelatedField):

    def to_representation(self, value):
        from content_studio.settings import cs_settings

        admin_site = cs_settings.ADMIN_SITE
        data = {"id": value.id, "__str__": str(value)}

        # Add file URL and media type if the model is a media library model.
        if value.__class__ is cs_settings.MEDIA_LIBRARY_MODEL:
            data["file"] = value.file.url
            data["type"] = value.type
            data["thumbnail"] = admin_site.get_thumbnail(value)

        return data

    def to_internal_value(self, data):
        return self.get_queryset().get(id=data["id"])


class ContentSerializer(serializers.ModelSerializer):
    __str__ = serializers.CharField(read_only=True)
    serializer_related_field = ContentRelatedField

    def get_field_names(self, declared_fields, info):
        """
        Override to automatically add model properties to the fields list.
        """
        # Get the default fields from parent class
        fields = super().get_field_names(declared_fields, info)

        # If fields is '__all__', we need to expand it
        if fields == "__all__":
            fields = list(super().get_field_names(declared_fields, info))
        else:
            fields = list(fields)

        # Get all properties from the model
        model_properties = self._get_model_properties()

        # Add properties that aren't already in fields
        for prop_name in model_properties:
            if prop_name not in fields and prop_name not in declared_fields:
                fields.append(prop_name)

        return fields

    def _get_model_properties(self):
        """
        Extract all @property attributes from the model class.
        """
        model = self.Meta.model
        properties = []

        # Inspect the model class and its parent classes
        for klass in inspect.getmro(model):
            if klass == object:
                break

            for name, obj in inspect.getmembers(klass):
                # Check if it's a property and not private/protected
                if isinstance(obj, property) and not name.startswith("_"):
                    if name not in properties:
                        properties.append(name)

        return properties

    def build_property_field(self, field_name, model_class):
        """
        Create a ReadOnlyField for property attributes.
        """
        return serializers.ReadOnlyField, {}

    def build_field(self, field_name, info, model_class, nested_depth):
        """
        Override to handle property fields.
        """
        # Check if this is a property
        if hasattr(model_class, field_name):
            attr = getattr(model_class, field_name)
            if isinstance(attr, property):
                return self.build_property_field(field_name, model_class)

        # Fall back to default behavior for regular fields
        return super().build_field(field_name, info, model_class, nested_depth)


class RelatedItemSerializer(serializers.Serializer):
    """
    Serializer for use in the relations endpoint.
    """

    id = serializers.UUIDField()
    __str__ = serializers.CharField()


class SessionUserSerializer(serializers.ModelSerializer):
    username = serializers.CharField(source=user_model.USERNAME_FIELD)

    class Meta:
        model = user_model
        fields = (
            "id",
            "username",
            "first_name",
            "last_name",
        )


class UserSerializer(serializers.ModelSerializer):

    class Meta:
        model = user_model
        fields = (
            "id",
            "first_name",
            "last_name",
        )



import operator
import uuid
from functools import reduce

from django.contrib.admin.models import LogEntry, ADDITION, CHANGE, DELETION
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Q
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound, ValidationError
from rest_framework.filters import SearchFilter, OrderingFilter
from rest_framework.parsers import JSONParser
from rest_framework.permissions import DjangoModelPermissions
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from .filters import LookupFilter
from .serializers import RelatedItemSerializer
from .settings import cs_settings
from .utils import get_tenant_field_name


class BaseModelViewSet(ModelViewSet):
    lookup_field = "id"
    is_singleton = False
    parser_classes = [JSONParser]
    renderer_classes = [JSONRenderer]
    permission_classes = [DjangoModelPermissions]
    filter_backends = [SearchFilter, OrderingFilter, LookupFilter]

    def __init__(self, *args, **kwargs):
        super(BaseModelViewSet, self).__init__()
        admin_site = cs_settings.ADMIN_SITE

        self.authentication_classes = [
            admin_site.token_backend.active_backend.authentication_class
        ]

    def list(self, request, *args, **kwargs):
        """
        We overwrite the list method to support singletons. If a singleton
        doesn't exist this will raise a NotFound exception.
        """
        if self.is_singleton:
            return super().retrieve(request, *args, **kwargs)

        return super().list(request, *args, **kwargs)

    def perform_create(self, serializer):
        tenant_id = self.request.headers.get("x-dcs-tenant", None)
        tenant_model = cs_settings.TENANT_MODEL
        model = self.get_queryset().model
        tenant_field_name = get_tenant_field_name(model)
        attrs = {}

        if tenant_model and tenant_id and tenant_field_name:
            attrs[f"{tenant_field_name}_id"] = tenant_id

        if hasattr(model, cs_settings.CREATED_BY_ATTR):
            attrs[cs_settings.CREATED_BY_ATTR] = self.request.user

        instance = serializer.save(**attrs)

        content_type = ContentType.objects.get_for_model(instance)
        LogEntry.objects.create(
            user=self.request.user,
            action_flag=ADDITION,
            content_type=content_type,
            object_id=instance.id,
            object_repr=str(instance)[:200],
            change_message="",
        )

    def perform_update(self, serializer):
        instance = serializer.save()

        if hasattr(instance, cs_settings.EDITED_BY_ATTR):
            setattr(instance, cs_settings.EDITED_BY_ATTR, self.request.user)
            instance.save()

        content_type = ContentType.objects.get_for_model(instance)
        LogEntry.objects.create(
            user=self.request.user,
            action_flag=CHANGE,
            content_type=content_type,
            object_id=instance.id,
            object_repr=str(instance)[:200],
            change_message="",
        )

    def perform_destroy(self, instance):
        content_type = ContentType.objects.get_for_model(instance)
        LogEntry.objects.create(
            user=self.request.user,
            action_flag=DELETION,
            content_type=content_type,
            object_id=instance.id,
            object_repr=str(instance)[:200],
            change_message="",
        )

        instance.delete()

    def get_object(self):
        """
        We overwrite this method to add support for singletons.
        If a singleton doesn't exist it will raise a NotFound exception.
        """
        if self.is_singleton:
            singleton = self.get_queryset().first()

            if singleton:
                return singleton
            else:
                raise NotFound()

        return super().get_object()

    @action(
        methods=["get"], detail=True, url_path="components/(?P<component_id>[^/.]+)"
    )
    def get_component(self, request, id, component_id):
        component = self._admin_model.get_component(uuid.UUID(component_id))

        if not component:
            raise NotFound()

        return component.handle_request(obj=self.get_object(), request=request)

    @action(methods=["post"], detail=False, url_path="relations/(?P<field_name>[^/]+)")
    def get_related_objects(self, request, field_name):
        """
        Endpoint for retrieving related objects.
        """
        search = request.data.get("search", "")

        parent_model = self.queryset.model

        try:
            related_field = parent_model._meta.get_field(field_name)
        except LookupError:
            raise ValidationError("Related field not found.")

        related_model = related_field.related_model

        custom_filter_method = getattr(
            self._admin_model, f"get_related_{field_name}", None
        )

        if custom_filter_method:
            qs = custom_filter_method(
                search=search,
                form_data=request.data.get("form", {}),
                related_model=related_model,
                request=request,
            )

        else:
            char_fields = [
                f.name
                for f in related_model._meta.get_fields()
                if isinstance(f, models.CharField)
            ]

            queries = [Q(**{f"{f}__icontains": search}) for f in char_fields]

            combined_query = reduce(operator.or_, queries)

            qs = related_model.objects.filter(combined_query)

        serializer = RelatedItemSerializer(qs[:20], many=True)

        return Response(data=serializer.data)



from django.urls import re_path

from .media_library.viewsets import MediaLibraryViewSet, MediaFolderViewSet
from .router import content_studio_router
from .views import ContentStudioWebAppView, AdminApiViewSet

content_studio_router.register("api", AdminApiViewSet, "content_studio_admin")
content_studio_router.register(
    "api/media-library/items", MediaLibraryViewSet, "content_studio_media_library_items"
)
content_studio_router.register(
    "api/media-library/folders",
    MediaFolderViewSet,
    "content_studio_media_library_folders",
)

urlpatterns = [
    re_path(
        "^(?!api).*$", ContentStudioWebAppView.as_view(), name="content_studio_web"
    ),
] + content_studio_router.urls



from rest_framework import serializers

from ..dashboard import BaseWidget


class ContentSerializer(serializers.Serializer):
    model = serializers.CharField(allow_blank=False, required=True)
    id = serializers.CharField(allow_blank=False, required=True)
    title = serializers.CharField(allow_blank=False, required=True)
    description = serializers.CharField(
        default="",
        allow_blank=True,
        required=False,
    )


class ContentListWidgetSerializer(serializers.Serializer):
    title = serializers.CharField(default="", allow_blank=True, required=False)
    description = serializers.CharField(
        default="",
        allow_blank=True,
        required=False,
    )
    content = serializers.ListField(child=ContentSerializer())


class ContentListWidget(BaseWidget):
    """
    Widget for showing a list of content.
    """

    name = "ContentListWidget"

    col_span = 2

    def get_data(self, request):
        raise NotImplementedError("You need to implement get_data for your widget.")



import uuid

from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound
from rest_framework.parsers import JSONParser
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet

from content_studio.settings import cs_settings


class Dashboard:
    """
    The Dashboard class is used to define the structure of the dashboard
    in Django Content Studio.
    """

    widgets = None

    def __init__(self, **kwargs):
        self.widgets = kwargs.get("widgets", [])

    def set_up_router(self):
        from content_studio.router import content_studio_router

        content_studio_router.register(
            "api/dashboard",
            DashboardViewSet,
            basename="content_studio_dashboard",
        )

    def serialize(self):
        return {
            "widgets": [
                {
                    "name": w.name,
                    "widget_id": w.widget_id,
                    "col_span": w.col_span,
                }
                for w in self.widgets
            ]
        }


class DashboardViewSet(ViewSet):
    parser_classes = [JSONParser]
    renderer_classes = [JSONRenderer]

    def __init__(self, *args, **kwargs):
        super(ViewSet, self).__init__()
        admin_site = cs_settings.ADMIN_SITE

        self.dashboard = admin_site.dashboard
        self.authentication_classes = [
            admin_site.token_backend.active_backend.authentication_class
        ]

    @action(detail=False, url_path="widgets/(?P<widget_id>[^/.]+)")
    def get(self, request, widget_id=None):
        widget = None

        for w in self.dashboard.widgets:
            if widget_id == str(w.widget_id):
                widget = w

        if not widget:
            raise NotFound()

        data = widget.get_data(request)

        if isinstance(data, serializers.Serializer):
            data.is_valid(raise_exception=True)
            data = data.data

        return Response(data=data)


class BaseWidget:
    col_span = 1
    widget_id = None

    def __init__(self):
        self.widget_id = self.widget_id or uuid.uuid4()


class SpacingWidget(BaseWidget):
    name = "SpacingWidget"

    def __init__(self, col_span=1):
        self.col_span = col_span
        super().__init__()



from rest_framework import serializers


class PasswordResetRequestSerializer(serializers.Serializer):
    email = serializers.EmailField()


class CodeValidationSerializer(serializers.Serializer):
    code = serializers.CharField(max_length=6)


class PasswordResetSubmissionSerializer(serializers.Serializer):
    code = serializers.CharField(max_length=6)
    email = serializers.EmailField()
    password = serializers.CharField(min_length=8)



from rest_framework import status, exceptions
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.parsers import JSONParser, MultiPartParser
from rest_framework.permissions import DjangoModelPermissions
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from content_studio.paginators import ContentPagination
from content_studio.settings import cs_settings
from .serializers import MediaItemSerializer, MediaFolderSerializer


class MediaLibraryViewSet(ModelViewSet):
    _media_model = None
    lookup_field = "id"
    parser_classes = [JSONParser, MultiPartParser]
    renderer_classes = [JSONRenderer]
    permission_classes = [DjangoModelPermissions]
    filter_backends = [SearchFilter, OrderingFilter]
    pagination_class = ContentPagination
    search_fields = ["name", "tags"]

    def __init__(self, *args, **kwargs):
        super(MediaLibraryViewSet, self).__init__()

        admin_site = cs_settings.ADMIN_SITE

        self.authentication_classes = [
            admin_site.token_backend.active_backend.authentication_class
        ]

    def get_queryset(self):
        if not self._media_model:
            self._media_model = cs_settings.MEDIA_LIBRARY_MODEL

        if not self._media_model:
            raise exceptions.MethodNotAllowed(
                method="GET", detail="Media model not defined."
            )

        folder = self.request.query_params.get("folder", None)
        search = self.request.query_params.get("search", None)
        qs = self._media_model.objects.all()

        if folder:
            if folder == "root":
                return qs.filter(folder__isnull=True)
            return qs.filter(folder=folder)

        return qs

    def get_serializer_class(self):
        if self._media_model:
            return MediaItemSerializer

        raise exceptions.MethodNotAllowed(
            method="GET", detail="Media model not defined."
        )


class MediaFolderViewSet(ModelViewSet):
    _folder_model = None
    lookup_field = "id"
    parser_classes = [JSONParser]
    renderer_classes = [JSONRenderer]
    permission_classes = [DjangoModelPermissions]
    filter_backends = [SearchFilter, OrderingFilter]
    pagination_class = ContentPagination

    def __init__(self, *args, **kwargs):
        super(MediaFolderViewSet, self).__init__()

        admin_site = cs_settings.ADMIN_SITE

        self.authentication_classes = [
            admin_site.token_backend.active_backend.authentication_class
        ]

    def get_queryset(self):
        if not self._folder_model:
            self._folder_model = cs_settings.MEDIA_LIBRARY_FOLDER_MODEL

        if not self._folder_model:
            raise exceptions.MethodNotAllowed(
                method="GET", detail="Media folder model not defined."
            )

        parent = self.request.query_params.get("parent", None)
        qs = self._folder_model.objects.all()

        if self.action != "list":
            return qs

        # The list endpoint is always within the scope of a folder
        if not parent:
            return qs.filter(parent__isnull=True)
        return qs.filter(parent=parent)

    def get_serializer_class(self):
        if self._folder_model:
            return MediaFolderSerializer

        raise exceptions.MethodNotAllowed(
            method="GET", detail="Media folder model not defined."
        )

    @action(methods=["get"], detail=False, url_path="path")
    def get(self, request, *args, **kwargs):

        if not self._folder_model:
            self._folder_model = cs_settings.MEDIA_LIBRARY_FOLDER_MODEL

        if not self._folder_model:
            raise exceptions.MethodNotAllowed(
                method="GET", detail="Folder model not defined."
            )

        folder_id = request.query_params.get("folder", None)

        if not folder_id:
            return Response(data=[])

        try:
            folder = self._folder_model.objects.get(pk=folder_id)
            path = []
            while folder:
                path.insert(0, folder)
                folder = folder.parent

            return Response(data=MediaFolderSerializer(path, many=True).data)
        except self._folder_model.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)



from .username_password import UsernamePasswordBackend
from ..router import content_studio_router
from ..settings import cs_settings


class LoginBackendManager:
    """
    Manages different login backends for use by
    Content Studio.
    """

    def __init__(self, **kwargs):
        self.active_backends = cs_settings.LOGIN_BACKENDS

    def set_up_router(self):
        for backend in self.active_backends:
            content_studio_router.register(
                f"api/login/{backend.__name__.lower().replace('backend', '')}",
                backend.view_set,
                basename="content_studio_login_backend",
            )



from decimal import Decimal

from django.db import models
from rest_framework.exceptions import ParseError
from rest_framework.filters import BaseFilterBackend

from .utils import flatten


class LookupFilter(BaseFilterBackend):
    """
    A permissive filter backend that basically allows every supported lookup
    for a given field. Automatically handles multi-value lookups and booleans.
    Also supports exclusion.
    """

    MULTI_VALUE_LOOKUPS = ["in", "range"]

    EXCLUDE_SYMBOL = "~"

    NON_FILTER_FIELDS = ["search", "limit", "page", "ordering"]

    def filter_queryset(self, request, queryset, view):
        """
        Build the queryset based on the query params and the view's model.
        Only apply filters in list endpoints.
        """
        if getattr(view, "action", None) == "list":
            try:
                filter_kwargs, exclude_kwargs = self.get_filter_kwargs(
                    model_class=view.queryset.model,
                    query_params=request.query_params,
                )
            except Exception as e:
                print(e)
                raise ParseError(detail="Invalid filter parameters")
            return queryset.filter(**filter_kwargs).exclude(**exclude_kwargs).distinct()

        return queryset

    def get_filter_kwargs(self, model_class, query_params):
        filter_kwargs = {}
        exclude_kwargs = {}

        field_lookups = self.get_field_lookups(model_class=model_class)

        for key, value in query_params.lists():
            if key in self.NON_FILTER_FIELDS:
                continue
            # By default Django supports repeated multi-values (e.g. `a=1&a=2`)
            # but we allow for comma-seperated multi-values as well (e.g. `a=1,2`).
            value = flatten([param.split(",") for param in value])
            is_exclude = key.startswith(self.EXCLUDE_SYMBOL)
            # The first part of a key is considered the field name
            field_name = key.split("__")[0]
            # Get the model field.
            field = model_class._meta.get_field(field_name)
            # Get the allowed lookups for this field.
            lookups = field_lookups.get(field_name, [])
            try:
                # The last part of a key is considered its lookup
                # but it's not required.
                lookup = key.split("__")[-1]
                if lookup not in lookups:
                    lookup = None
            except IndexError:
                lookup = None

            # Some lookups allow multiple values, otherwise
            # the first value is used.
            is_multi = lookup in self.MULTI_VALUE_LOOKUPS
            # Depending on the field type we cast the value to
            # its correct type (i.e. number, boolean, etc.).
            if is_multi:
                casted_value = [self.cast_field_value(v, field) for v in value]
            elif lookup == "isnull":
                casted_value = value[0] in ["1", "true", "on"]
            else:
                casted_value = self.cast_field_value(value[0], field)

            if is_exclude:
                exclude_kwargs[key] = casted_value
            else:
                filter_kwargs[key] = casted_value

        return filter_kwargs, exclude_kwargs

    @staticmethod
    def get_field_lookups(model_class):
        """
        Allow all supported lookups.
        """
        field_lookups = {}
        for model_field in model_class._meta.get_fields():
            lookup_list = model_field.get_lookups().keys()
            field_lookups[model_field.name] = lookup_list
        return field_lookups

    def cast_field_value(self, value: str, field):
        value = value.strip().lower()

        if isinstance(field, models.BooleanField):
            if value in ["1", "true", "on"]:
                return True
            if value in ["0", "false", "off"]:
                return False
        if isinstance(field, models.NullBooleanField):
            if value in ["null", "none", "empty"]:
                return None

        if isinstance(field, models.IntegerField):
            return int(value)

        if isinstance(field, models.DecimalField):
            return Decimal(value)

        if isinstance(field, models.FloatField):
            return float(value)

        return value



from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework_simplejwt.settings import api_settings as simplejwt_settings
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework_simplejwt.views import TokenRefreshView


class SimpleJwtViewSet(ViewSet):
    @action(
        detail=False, methods=["post"], permission_classes=[], authentication_classes=[]
    )
    def refresh(self, request):

        view_instance = TokenRefreshView()
        view_instance.request = request
        view_instance.format_kwarg = None
        return view_instance.post(request)


class SimpleJwtBackend:
    name = "Simple JWT"
    authentication_class = JWTAuthentication
    view_set = SimpleJwtViewSet

    @classmethod
    def get_info(cls):

        return {
            "type": cls.__name__,
            "config": {
                "ACCESS_TOKEN_LIFETIME": simplejwt_settings.ACCESS_TOKEN_LIFETIME.total_seconds(),
            },
        }

    @property
    def is_available(self) -> bool:
        try:
            import rest_framework_simplejwt

            return True
        except ImportError:
            return False

    @classmethod
    def get_response_for_user(cls, user):

        refresh = RefreshToken.for_user(user)

        return Response(
            {
                "refresh": str(refresh),
                "access": str(refresh.access_token),
            }
        )



import uuid
from typing import Type

from blueprint import fields as bp_fields
from blueprint.media_library.fields import MediaField, ManyMediaField
from django.contrib import admin
from django.db import models
from django.db.models import Model
from rest_framework.request import HttpRequest

from . import widgets, formats
from .form import FormSet, FormSetGroup, Field, Component
from .login_backends import LoginBackendManager
from .token_backends import TokenBackendManager
from .utils import get_related_field_name, flatten

register = admin.register
display = admin.display


class StackedInline(admin.StackedInline):
    pass


class TabularInline(admin.TabularInline):
    pass


class AdminSite(admin.AdminSite):
    """
    Enhanced admin site for Django Content Studio.
    """

    token_backend = TokenBackendManager()

    login_backend = LoginBackendManager()

    dashboard = None

    model_groups = None

    extensions = None

    default_widget_mapping = {
        models.CharField: widgets.InputWidget,
        models.IntegerField: widgets.InputWidget,
        models.SmallIntegerField: widgets.InputWidget,
        models.BigIntegerField: widgets.InputWidget,
        models.PositiveIntegerField: widgets.InputWidget,
        models.PositiveSmallIntegerField: widgets.InputWidget,
        models.PositiveBigIntegerField: widgets.InputWidget,
        models.FloatField: widgets.InputWidget,
        models.DecimalField: widgets.InputWidget,
        models.SlugField: widgets.SlugWidget,
        models.TextField: widgets.TextAreaWidget,
        models.BooleanField: widgets.CheckboxWidget,
        models.NullBooleanField: widgets.CheckboxWidget,
        models.ForeignKey: widgets.ForeignKeyWidget,
        models.ManyToManyField: widgets.ManyToManyWidget,
        models.OneToOneField: widgets.ForeignKeyWidget,
        models.DateField: widgets.DateWidget,
        models.DateTimeField: widgets.DateTimeWidget,
        models.TimeField: widgets.TimeWidget,
        models.JSONField: widgets.JSONWidget,
        # Blueprint fields
        bp_fields.HTMLField: widgets.RichTextWidget,
        bp_fields.TagField: widgets.TagWidget,
        bp_fields.FlexField: widgets.JSONSchemaWidget,
        bp_fields.MultipleChoiceField: widgets.MultiSelectWidget,
        bp_fields.URLPathField: widgets.URLPathWidget,
        MediaField: widgets.MediaWidget,
        ManyMediaField: widgets.ManyMediaWidget,
        # Common third-party fields
        "AutoSlugField": widgets.SlugWidget,
    }

    default_format_mapping = {
        models.CharField: formats.TextFormat,
        models.IntegerField: formats.NumberFormat,
        models.SmallIntegerField: formats.NumberFormat,
        models.BigIntegerField: formats.NumberFormat,
        models.PositiveIntegerField: formats.NumberFormat,
        models.PositiveSmallIntegerField: formats.NumberFormat,
        models.PositiveBigIntegerField: formats.NumberFormat,
        models.FloatField: formats.NumberFormat,
        models.DecimalField: formats.NumberFormat,
        models.SlugField: formats.TextFormat,
        models.TextField: formats.TextFormat,
        models.BooleanField: formats.BooleanFormat,
        models.NullBooleanField: formats.BooleanFormat,
        models.DateField: formats.DateFormat,
        models.DateTimeField: formats.DateTimeFormat,
        models.TimeField: formats.TimeFormat,
        models.ForeignKey: formats.ForeignKeyFormat,
        models.OneToOneField: formats.ForeignKeyFormat,
        models.JSONField: formats.JSONFormat,
        # Blueprint fields
        bp_fields.HTMLField: formats.HtmlFormat,
        bp_fields.TagField: formats.TagFormat,
        bp_fields.FlexField: formats.JSONFormat,
        bp_fields.MultipleChoiceField: formats.TextFormat,
        bp_fields.URLPathField: formats.TextFormat,
        MediaField: formats.MediaFormat,
        ManyMediaField: formats.ManyMediaFormat,
    }

    def setup(self):
        # Add token backend's view set to the
        # Content Studio router.
        self.token_backend.set_up_router()
        # Add login backend's view set to the
        # Content Studio router.
        self.login_backend.set_up_router()
        # Add dashboard's view set to the
        # Content Studio router.
        if self.dashboard:
            self.dashboard.set_up_router()

    def get_thumbnail(self, obj) -> str:
        """
        Method for getting and manipulating the image path (or URL).
        By default, this returns the image path as is.
        """
        return obj.file.url

    def get_tenants(self, tenant_model: Type[models.Model], **kwargs):
        """
        Method for getting the list of available tenants.
        """
        return tenant_model.objects.all()


admin_site = AdminSite()


class ModelAdmin(admin.ModelAdmin):
    """
    Enhanced model admin for Django Content Studio and integration with
    Django Content Framework. Although it's relatively backwards compatible,
    some default behavior has been changed.
    """

    # Whether the model is a singleton and should not show
    # the list view.
    is_singleton = False

    # Override the widget used for certain fields by adding
    # a map of field to widget. Fields that are not included
    # will fall back to their default widget.
    #
    # @example
    # widget_mapping = {'is_published': widgets.SwitchWidget}
    widget_mapping = None

    # Override the format used for certain fields by adding
    # a map of field to format. Fields that are not included
    # will fall back to their default format.
    #
    # @example
    # format_mapping = {'file_size': widgets.FileSizeWidget}
    format_mapping = None

    # We set a lower limit than Django's default of 100
    list_per_page = 20

    # Description shown below model name on list pages
    list_description = ""

    # Configure the main section in the edit-view.
    edit_main: list[type[FormSetGroup | FormSet | Field | str]] = []

    # Configure the sidebar in the edit-view.
    edit_sidebar: list[type[FormSet | Field | str]] = []

    icon = None

    def has_add_permission(self, request):
        is_singleton = getattr(self.model, "is_singleton", False)

        # Don't allow to add more than one singleton object.
        if is_singleton and self.model.objects.get():
            return False

        return super().has_add_permission(request)

    def has_delete_permission(self, request, obj=None):
        is_singleton = getattr(self.model, "is_singleton", False)

        if is_singleton:
            return False

        return super().has_delete_permission(request, obj)

    def get_component(self, component_id: uuid.UUID):
        """
        Retrieves a component from the edit_main or edit_sidebar attributes by ID.
        :param component_id:
        :return:
        """
        all_fields = getattr(self, "edit_main", []) + getattr(self, "edit_sidebar", [])

        flat_fields = flatten(
            [f.get_fields() if hasattr(f, "get_fields") else [f] for f in all_fields]
        )

        components = [c for c in flat_fields if issubclass(c.__class__, Component)]

        for component in components:
            if component.component_id == component_id:
                return component

        return None


class AdminSerializer:
    """
    Class for serializing Django admin classes.
    """

    def __init__(self, admin_class: ModelAdmin):
        self.admin_class = admin_class

    def serialize(self, request: HttpRequest):
        admin_class = self.admin_class
        format_mapping = getattr(admin_class, "format_mapping", None) or {}
        widget_mapping = getattr(admin_class, "widget_mapping", None) or {}

        return {
            "icon": getattr(admin_class, "icon", None),
            "is_singleton": getattr(admin_class, "is_singleton", False),
            "edit": {
                "main": self.serialize_edit_main(request),
                "sidebar": self.serialize_edit_sidebar(request),
                "inlines": [
                    {
                        "model": inline.model._meta.label_lower,
                        "fk_name": get_related_field_name(inline, admin_class.model),
                        "list_display": getattr(inline, "list_display", None)
                        or ["__str__"],
                    }
                    for inline in admin_class.inlines
                ],
            },
            "list": {
                "per_page": admin_class.list_per_page,
                "description": getattr(admin_class, "list_description", ""),
                "display": self.get_list_display(),
                "search": len(admin_class.search_fields) > 0,
                "filter": admin_class.list_filter,
                "sortable_by": admin_class.sortable_by,
            },
            "widget_mapping": {
                field: widget.serialize() for field, widget in widget_mapping.items()
            },
            "format_mapping": {
                field: format.serialize() for field, format in format_mapping.items()
            },
            "permissions": {
                "add_permission": admin_class.has_add_permission(request),
                "delete_permission": admin_class.has_delete_permission(request),
                "change_permission": admin_class.has_change_permission(request),
                "view_permission": admin_class.has_view_permission(request),
            },
        }

    def serialize_edit_main(self, request):
        admin_class = self.admin_class

        return [
            i.serialize()
            for i in self.get_edit_main(
                getattr(admin_class, "edit_main", admin_class.get_fields(request))
            )
        ]

    def serialize_edit_sidebar(self, request):
        admin_class = self.admin_class

        return [
            i.serialize()
            for i in self.get_edit_sidebar(getattr(admin_class, "edit_sidebar", None))
        ]

    def get_list_display(self):
        admin_class = self.admin_class
        fields = []

        for field in admin_class.list_display:
            if hasattr(admin_class, field):
                method = getattr(admin_class, field)
                description = getattr(method, "short_description", None)
                empty_value = getattr(method, "empty_value", None)
                fields.append(
                    {
                        "name": field,
                        "description": description,
                        "empty_value": empty_value,
                    }
                )
            else:
                fields.append({"name": field})

        return fields

    def get_edit_main(self, edit_main):
        """
        Returns a normalized list of form set groups.

        Form sets will be wrapped in a form set group. If the edit_main attribute is a list of fields,
        they are wrapped in a form set and a form set group.
        """
        if not edit_main:
            return []
        if isinstance(edit_main[0], FormSetGroup):
            return edit_main
        if isinstance(edit_main[0], FormSet):
            return [FormSetGroup(formsets=edit_main)]

        return [FormSetGroup(formsets=[FormSet(fields=edit_main)])]

    def get_edit_sidebar(self, edit_sidebar):
        """
        Returns a normalized list of form sets for the edit_sidebar.

        If the edit_sidebar attribute is a list of fields,
        they are wrapped in a form set.
        """
        if not edit_sidebar:
            return []
        if isinstance(edit_sidebar[0], FormSet):
            return edit_sidebar

        return [FormSet(fields=edit_sidebar)]


class ModelGroup:
    name = None
    label = None
    icon = None
    color = None
    models = None

    def __init__(
        self,
        name: str,
        label: str = None,
        icon: str = None,
        color: str = None,
        models: list[Type[Model]] = None,
    ):
        self.name = name
        self.label = label or name.capitalize()
        self.icon = icon
        self.color = color
        self.models = models or []



from django.db import models

from .utils import is_jsonable, get_tenant_field_name


class ModelSerializer:
    def __init__(self, model: type[models.Model]):
        self.model = model

    def serialize(self):
        model = self.model

        tenant_field = get_tenant_field_name(self.model)

        return {
            "label": model._meta.label_lower,
            "verbose_name": model._meta.verbose_name,
            "verbose_name_plural": model._meta.verbose_name_plural,
            "fields": self.get_fields(),
            "tenant_field": tenant_field,
        }

    def get_fields(self):
        fields = {
            "__str__": {
                "type": "CharField",
                "readonly": True,
            }
        }
        standard_fields = self.model._meta.fields
        m2m_fields = self.model._meta.many_to_many

        for field in standard_fields + m2m_fields:
            fields[field.name] = self.get_field(field)

        return fields

    def get_field(self, field):
        data = {
            "verbose_name": field.verbose_name,
            "required": not field.null or not field.blank,
            "type": field.__class__.__name__,
        }

        if hasattr(field, "widget_class"):
            data["widget_class"] = field.widget_class.__name__

        if hasattr(field, "format_class"):
            data["format_class"] = field.format_class.__name__

        if field.help_text:
            data["help_text"] = field.help_text

        if is_jsonable(field.default):
            data["default"] = field.default

        if not field.editable:
            data["readonly"] = True

        if field.primary_key:
            data["primary_key"] = True
            data["readonly"] = True

        if field.is_relation:
            data["related_model"] = field.related_model._meta.label_lower

        if getattr(field, "choices", None) is not None:
            data["choices"] = field.choices

        if getattr(field, "max_length", None) is not None:
            data["max_length"] = field.max_length

        if getattr(field, "cs_get_field_attributes", None):
            data.update(field.cs_get_field_attributes())

        return data



###
# Widgets determine what input field is used for a certain
# model field. Each Django model field has a default widget,
# but this can be overridden in the admin model.
###


class BaseWidget:
    @classmethod
    def serialize(cls):
        return {"name": cls.__name__}


class InputWidget(BaseWidget):
    pass


class TextAreaWidget(BaseWidget):
    pass


class SwitchWidget(BaseWidget):
    pass


class CheckboxWidget(BaseWidget):
    pass


class DateWidget(BaseWidget):
    pass


class DateTimeWidget(BaseWidget):
    pass


class TimeWidget(BaseWidget):
    pass


class RichTextWidget(BaseWidget):
    pass


class RadioButtonWidget(BaseWidget):
    pass


class SelectWidget(BaseWidget):
    pass


class MultiSelectWidget(BaseWidget):
    pass


class URLPathWidget(BaseWidget):
    pass


class SlugWidget(BaseWidget):
    pass


class ForeignKeyWidget(BaseWidget):
    pass


class ManyToManyWidget(BaseWidget):
    pass


class JSONSchemaWidget(BaseWidget):
    pass


class JSONWidget(BaseWidget):
    pass


class MediaWidget(BaseWidget):
    pass


class ManyMediaWidget(BaseWidget):
    pass


class TagWidget(BaseWidget):
    pass



from django.apps import AppConfig
from django.contrib import admin

from . import VERSION
from .paginators import ContentPagination
from .settings import cs_settings
from .utils import is_runserver, get_tenant_field_name


class DjangoContentStudioConfig(AppConfig):
    name = "content_studio"
    label = "content_studio"
    initialized = False

    def ready(self):
        from .utils import log

        if is_runserver() and not self.initialized:
            self.initialized = True

            log("\n")
            log("----------------------------------------")
            log("Django Content Studio")
            log(f"Version {VERSION}")
            log("----------------------------------------")
            log(":rocket:", "Starting Django Content Studio")
            log(":mag:", "Discovering admin models...")
            registered_models = len(admin.site._registry)
            log(
                ":white_check_mark:",
                f"[green]Found {registered_models} admin models[/green]",
            )
            # Set up admin site routes
            admin_site = cs_settings.ADMIN_SITE
            admin_site.setup()

            # Set up content CRUD APIs
            self._create_crud_api()

            log("\n")

    def _create_crud_api(self):
        from .utils import log

        for model, admin_model in admin.site._registry.items():
            self._create_view_set(model, admin_model)

            for inline in admin_model.inlines:
                self._create_view_set(
                    parent=model, model=inline.model, admin_model=inline
                )

        log(
            ":white_check_mark:",
            f"[green]Created CRUD API[/green]",
        )

    def _create_view_set(self, model, admin_model, parent=None):
        from .viewsets import BaseModelViewSet
        from .router import content_studio_router
        from .serializers import ContentSerializer

        class Pagination(ContentPagination):
            page_size = getattr(admin_model, "list_per_page", 10)

        class ViewSet(BaseModelViewSet):
            _model = model
            _admin_model = admin_model
            is_singleton = getattr(admin_model, "is_singleton", False)
            pagination_class = Pagination
            queryset = _model.objects.none()
            search_fields = list(getattr(_admin_model, "search_fields", []))

            def get_serializer_class(self):
                # For list views we include the specified list_display fields.
                if self.action == "list" and not self.is_singleton:
                    available_fields = [
                        "id",
                        "__str__",
                    ] + list(getattr(self._admin_model, "list_display", []))
                # In all other cases we include all fields.
                else:
                    available_fields = "__all__"

                class Serializer(ContentSerializer):

                    class Meta:
                        model = self._model
                        fields = available_fields

                return Serializer

            def get_queryset(self):
                tenant_model = cs_settings.TENANT_MODEL
                tenant_id = self.request.headers.get("x-dcs-tenant", None)
                field_name = get_tenant_field_name(self._model)

                if tenant_model and tenant_id and field_name:
                    return self._model.objects.filter(**{f"{field_name}_id": tenant_id})

                return self._model.objects.all()

        if parent:
            prefix = f"api/inlines/{parent._meta.label_lower}/{model._meta.label_lower}"
            basename = f"content_studio_api-{parent._meta.label_lower}-{model._meta.label_lower}"
        else:
            prefix = f"api/content/{model._meta.label_lower}"
            basename = f"content_studio_api-{model._meta.label_lower}"

        content_studio_router.register(prefix, ViewSet, basename)
